from PyQt5.QtWidgets import *

import time
import cv2
from scipy.interpolate import UnivariateSpline
import matplotlib.pyplot as plt
import numpy as np
# import math
import pandas as pd
from cydets.algorithm import detect_cycles
import ctypes
import scipy.optimize as op
from scipy import signal
from PyQt5.QtCore import  QThread, pyqtSignal

class Plotter:
    def __init__(self, plot_width, plot_height, sample_buffer=None):
        self.width = plot_width
        self.height = plot_height
        self.color = (255, 0, 0)
        self.plot_canvas = np.zeros((self.height, self.width, 3), dtype=np.uint8)
        self.ltime = 0
        self.plots = {}
        self.plot_t_last = {}
        self.margin_l = 10
        self.margin_r = 10
        self.margin_u = 10
        self.margin_d = 50
        self.sample_buffer = self.width if sample_buffer is None else sample_buffer

    # Update new values in plot. Retrun result chart nd array.
    def plot(self, val, label="plot"):
        if not label in self.plots:
            self.plots[label] = []
            self.plot_t_last[label] = 0

        self.plots[label].append(int(val))

        while len(self.plots[label]) > self.sample_buffer:
            self.plots[label].pop(0)
            result_val = self.show_plot(label)
            return result_val
        return self.plot_canvas

    # Return result cart plot. nd.array
    def show_plot(self, label):
        self.plot_canvas = np.zeros((self.height, self.width, 3), dtype=np.uint8)
        cv2.line(self.plot_canvas, (self.margin_l, int((self.height-self.margin_d-self.margin_u)/2)+self.margin_u),
                    (self.width-self.margin_r, int((self.height-self.margin_d-self.margin_u)/2)+self.margin_u), (0, 0, 255), 1)

        # Scaling the graph in y within buffer
        scale_h_max = max(self.plots[label])
        scale_h_min = min(self.plots[label])
        scale_h_min = -scale_h_min if scale_h_min < 0 else scale_h_min
        scale_h = scale_h_max if scale_h_max > scale_h_min else scale_h_min
        scale_h = ((self.height-self.margin_d-self.margin_u)/2) / \
            scale_h if not scale_h == 0 else 0

        for j, i in enumerate(np.linspace(0, self.sample_buffer-2, self.width-self.margin_l-self.margin_r)):
            i = int(i)
            cv2.line(self.plot_canvas, (j+self.margin_l, int((self.height-self.margin_d-self.margin_u)/2 + self.margin_u - self.plots[label][i]*scale_h)), (
                j+self.margin_l, int((self.height-self.margin_d-self.margin_u)/2 + self.margin_u - self.plots[label][i+1]*scale_h)), self.color, 1)

        cv2.rectangle(self.plot_canvas, (self.margin_l, self.margin_u),
                        (self.width-self.margin_r, self.height-self.margin_d), (255, 255, 255), 1)
        cv2.putText(self.plot_canvas, f" {label} : {self.plots[label][-1]} , dt : {int((time.time() - self.plot_t_last[label])*1000)}ms", (int(
            0), self.height-20), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (0, 255, 255), 2)
        cv2.circle(self.plot_canvas, (self.width-self.margin_r, int(self.margin_u + (self.height -
                    self.margin_d-self.margin_u)/2 - self.plots[label][-1]*scale_h)), 2, (0, 200, 200), -1)

        self.plot_t_last[label] = time.time()

        return self.plot_canvas
        
class HeartRate(QThread):
    # Main signals of the QThread.
    sin_final_video_frame = pyqtSignal(np.ndarray)      # Send main video frame to the main App class.
    sin_crop_video_1 = pyqtSignal(np.ndarray)           # Send first video frame to the main App class.
    sin_crop_video_2 = pyqtSignal(np.ndarray)           # Send main video frame to the main App class.

    sin_result_chart_1 = pyqtSignal(np.ndarray)         # Send result chart numpy array to the main App class.
    sin_result_chart_2 = pyqtSignal(np.ndarray)         # Send result chart numpy array to the main App class.

    sin_videoFileName = pyqtSignal(str)                 # Received the video file name from main App class.
    sin_runningFlag = pyqtSignal(int)                   # Received running flag from main App class. 
    sin_roi_area_1 = pyqtSignal(tuple)                  # first area (x, y, w, h)
    sin_roi_area_2 = pyqtSignal(tuple)                  # second area (x, y, w, h)

    sin_atrium = pyqtSignal(str)                        # Send atrium to the main App class.
    sin_ventricle = pyqtSignal(str)                     # Send ventricle to the main App class.

    sin_fishTyep = pyqtSignal(str)                      # Received the fish type from main App class.

    sin_end_frame = pyqtSignal(int)                     # Send end signal to the main App class.
    sin_frame_count = pyqtSignal(list)            

    def __init__(self):
        super(HeartRate,self).__init__()
        # Main parameters
        self.running_flag = 0
        self.video_file_name = ''
        self.fish_type = 'fish'
        # selected area tuple
        self.roi_rectAngle_1 = ()
        self.roi_rectAngle_2 = ()

        self.index = 0
        self.indexFrm = 0
        self.myPlot = Plotter(400, 200, sample_buffer=200)

        self.xlist = list()
        self.xlist2 = list()
        self.ylist = list()
        self.ylist2 = list()
        self.frameNo = 0

        # heart rate data
        self.first_Image = None
        self.first_Image2 = None
        self.maxcl = 0
        self.maxcl2 = 0
        self.pplotpix = []
        self.pplotpix2 = []
        self.index_per_cycle = 0
        self.index_per_cycle2 = 0
        self.rate_per_ten = list()
        self.rate_per_ten2 = list()
        self.heart_rate_realtime = list()
        self.heart_rate_realtime2 = list()

        # Thread siganl and connect
        self.sin_videoFileName.connect(self.setVideoFileName)
        self.sin_runningFlag.connect(self.setRunningFlag)
        self.sin_roi_area_1.connect(self.setROIArea1)
        self.sin_roi_area_2.connect(self.setROIArea2)
        self.sin_fishTyep.connect(self.setFishType)
        
    def Mbox(self, title, text, style):
        return ctypes.windll.user32.MessageBoxW(0, text, title, style)

    def setVideoFileName(self, video_file_name):
        print("Update video file name!")
        self.video_file_name = video_file_name

    def setRunningFlag(self, flag):
        self.index = 0
        self.indexFrm = 0

        self.running_flag = flag
        if self.running_flag == 1:
            self.xlist = list()
            self.xlist2 = list()
            self.ylist = list()
            self.ylist2 = list()
            self.frameNo = 0
            
            # heart rate data
            self.first_Image = None
            self.first_Image2 = None
            self.maxcl = 0
            self.maxcl2 = 0
            self.pplotpix = []
            self.pplotpix2 = []
            self.index_per_cycle = 0
            self.index_per_cycle2 = 0
            self.rate_per_ten = list()
            self.rate_per_ten2 = list()
            self.heart_rate_realtime = list()
            self.heart_rate_realtime2 = list()
        

    def setROIArea1(self, area):
        self.roi_rectAngle_1 = area
    def setROIArea2(self, area):
        self.roi_rectAngle_2 = area

    def setFishType(self, fish_type):
        self.fish_type = fish_type

    def normalize_array(self, array):
        # ** for array
        average_of_array = np.mean(array)
        std_dev = np.std(array)
        for i in range(len(array)):
            array[i] = ((array[i] - average_of_array)/std_dev)
        return array

    def std_normalize(self, array):
        # ** for array
        min_value = np.min(array)
        max_value = np.max(array)
        scaled_array = (array - min_value) / (max_value - min_value)
        return scaled_array

    # Cross-Correlation algorithm
    def get_similarity(self, image1, image2):
        gray_image = cv2.cvtColor(image1, cv2.COLOR_BGR2GRAY)
        gray_image2 = cv2.cvtColor(image2, cv2.COLOR_BGR2GRAY)

        mat1 = np.asfarray(gray_image, dtype='float').flatten()
        mat2 = np.asfarray(gray_image2, dtype='float').flatten()

        rSum = 0.0
        rSum1 = 0.0
        rSum2 = 0.0

        for i in range(len(mat1)):
            rSum += (mat1[i]*mat2[i])**2
            rSum1 += (mat1[i])**2
            rSum2 += (mat2[i])**2

        rSum /= (rSum1 * rSum2)**0.5

        return rSum

    def findPeak(self, signal_arr):
        dary = signal_arr
        dary -= np.average(dary)

        step = np.hstack((np.ones(len(dary)), -1*np.ones(len(dary))))

        dary_step = np.convolve(dary, step, mode='valid')

        # Get the peaks of the convolution
        peaks = signal.find_peaks(dary_step, width=3)[0]
        return peaks

    def low_passfilter(self, data, fps_video):
        # Select low and high cutoff frequency
        fc_low = 0.1/fps_video
        fc_high = 0.4/fps_video
        b, a = signal.butter(1, [fc_low*2, fc_high*2], btype='bandpass')
        bandpass_filtered = signal.filtfilt(b, a, data)    
        return bandpass_filtered

    def get_heartrate(self, peaks_list, real_fps):
        cycle_interval = []
        cycle = 0
        for ii in range(len(peaks_list)):
            if ii < len(peaks_list)-1:
                cycle_interval.append(peaks_list[ii+1]-peaks_list[ii])
                cycle += (peaks_list[ii+1]-peaks_list[ii])
        # Calculate avg heartrate
        cycle = cycle/(len(peaks_list)-1)
        heatrate = real_fps/cycle * 60

        # Calculate each heartrate
        cycle_interval = np.array(cycle_interval)
        each_heartrate = real_fps/cycle_interval * 60
        return heatrate, cycle_interval, each_heartrate

    def get_interval_1(self, Atrium_peak = [], Ventricle_peak = [], cof = 1):
        interval_Atrium = []
        interval_Ventricle = []
        if len(Ventricle_peak) == len(Atrium_peak):
            for i in range(len(Atrium_peak)-1):
                Atrium  = (Atrium_peak[i+1] - Atrium_peak[i])*cof
                interval_Atrium.append(Atrium)
                Ventricle  = (Ventricle_peak[i+1] - Ventricle_peak[i])*cof
                interval_Ventricle.append(Ventricle)

            final = {
                'Atrium':             interval_Atrium,
                'Ventricle':          interval_Ventricle,
            }

            
            df_final = pd.DataFrame(data=final)
            df_final.to_csv('interval_1.csv')

    def get_interval_2(self, Atrium_peak = [], Ventricle_peak = [], cof = 1):
        interval_Atrium = []
        interval_Ventricle = []
        for i in range(len(Atrium_peak)-1):
            if Atrium_peak[i] < 0 or Ventricle_peak[i] < 0 or Atrium_peak[i+1] < 0 or Ventricle_peak[i+1] < 0:
                interval_Atrium.append(0)
                interval_Ventricle.append(0)
            else:
                if Atrium_peak[i] > Ventricle_peak[i]:
                    Atrium  = (Ventricle_peak[i+1] - Atrium_peak[i])/cof
                    interval_Atrium.append(Atrium)
                else:
                    Atrium  = (Ventricle_peak[i] - Atrium_peak[i])/cof
                    interval_Atrium.append(Atrium)

                if Atrium_peak[i+1] > Ventricle_peak[i+1]:
                    Ventricle  = (Atrium_peak[i+1] - Ventricle_peak[i+1])/cof
                    interval_Ventricle.append(Ventricle)
                else:
                    Ventricle  = (Atrium_peak[i+1] - Ventricle_peak[i])/cof
                    interval_Ventricle.append(Ventricle)

        a_A = sum(interval_Atrium)/ len(interval_Atrium)
        a_V = sum(interval_Ventricle)/ len(interval_Ventricle)
        
        interval_Atrium.append(' ')
        interval_Ventricle.append(' ')
        interval_Atrium.append(' ')
        interval_Ventricle.append(' ')
        
        interval_Atrium.append('A_V')
        interval_Ventricle.append('V_A')
        
        interval_Atrium.append(' ')
        interval_Ventricle.append(' ')
        interval_Atrium.append(' ')
        interval_Ventricle.append(' ')
        
        interval_Atrium.append(a_A)
        interval_Ventricle.append(a_V)
        

        final = {
            'Atrium':             interval_Atrium,
            'Ventricle':          interval_Ventricle,
        }

    
        df_final = pd.DataFrame(data=final)
        df_final.to_csv('interval_2.csv')

    def get_interval_daphnia(self, Atrium_peak = [], cof = 1):
        interval_Atrium = []
        for i in range(len(Atrium_peak)-1):
            Atrium  = (Atrium_peak[i+1] - Atrium_peak[i])*cof
            interval_Atrium.append(Atrium)

        final = {
            'Atrium':             interval_Atrium,
        }

        df_final = pd.DataFrame(data=final)
        df_final.to_csv('interval_daphnia.csv')
    
    def main_fish(self, frame, fps_video, real_fps, area1, area2, ratio = 1.0):
        
        (x, y, w, h) = area1
        crop_img = frame[y:y + h, x:x + w]
        self.sin_crop_video_1.emit(crop_img)
        
        (x2, y2, w2, h2) = area2
        crop_img2 = frame[y2:y2 + h2, x2:x2 + w2]
        self.sin_crop_video_2.emit(crop_img2)
        
        cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
        cv2.rectangle(frame, (x2, y2), (x2 + w2, y2 + h2), (0, 255, 0), 2)
        self.sin_final_video_frame.emit(frame)

        if(int(self.index) == 0):
            self.first_Image = crop_img
            self.first_Image2 = crop_img2

        self.index += 1
        if self.index < fps_video :
            gray_image = cv2.cvtColor(crop_img, cv2.COLOR_BGR2GRAY)
            gray_image2 = cv2.cvtColor(crop_img2, cv2.COLOR_BGR2GRAY)
            Binary = cv2.adaptiveThreshold(
                gray_image, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 11, 2)
            Binary2 = cv2.adaptiveThreshold(
                gray_image2, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 11, 2)
            cl = np.average(Binary)
            cl2 = np.average(Binary2)

            if cl > self.maxcl:
                self.maxcl = cl
                self.first_Image = crop_img
                text = "{}: {}".format(self.index+1, cl)
                cv2.putText(crop_img, text, (10, 20), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (0, 0, 255), 2)
            if cl2 > self.maxcl2:
                self.maxcl2 = cl2
                self.first_Image2 = crop_img2
                text2 = "{}: {}".format(self.index+1, cl2)
                cv2.putText(crop_img2, text2, (10, 20),  cv2.FONT_HERSHEY_SIMPLEX, 0.6, (0, 0, 255), 2)
        else:
            cl = self.get_similarity(self.first_Image, crop_img)
            cl2 = self.get_similarity(self.first_Image2, crop_img2)
            self.pplotpix.append(cl)
            self.pplotpix2.append(cl2)

            self.sin_result_chart_1.emit(self.myPlot.plot(cl, label='result'))
            self.sin_result_chart_2.emit(self.myPlot.plot(cl2, label='result'))
            
            self.indexFrm += 1
            if self.indexFrm >= fps_video and self.indexFrm % int(fps_video) == 0:
                y_data = np.array(self.pplotpix)
                x_data = np.arange(0, len(y_data))
                y_data = self.normalize_array(y_data)
                s = UnivariateSpline(x_data, y_data, s=50)
                yy_data = s(x_data)
                peaks = self.findPeak(yy_data)

                y_data2 = np.array(self.pplotpix2)
                x_data2 = np.arange(0, len(y_data2))
                y_data2 = self.normalize_array(y_data2)
                s2 = UnivariateSpline(x_data2, y_data2, s=50)
                yy_data2 = s2(x_data2)
                peaks2 = self.findPeak(yy_data2)

                heatrate = 0
                if(len(peaks) > 1):
                    heatrate = self.get_heartrate(peaks, real_fps)[0]
        
                if(heatrate != 0):
                    self.index_per_cycle += 1
                    # if index_per_cycle % 10 == 0:
                    self.rate_per_ten.append(heatrate)
                    self.heart_rate_realtime.append(heatrate)
                    self.sin_atrium.emit('{:.3f}'.format(heatrate))

                heatrate2 = 0
                if(len(peaks2) > 1):
                    heatrate2 = self.get_heartrate(peaks2, real_fps)[0]

                if(heatrate2 != 0):
                    self.index_per_cycle2 += 1
                    self.rate_per_ten2.append(heatrate2)
                    self.heart_rate_realtime2.append(heatrate2)
                    self.sin_ventricle.emit('{:.3f}'.format(heatrate2))

    def result_fish(self, fps_video, real_fps, ratio = 1.0):
        y_data = np.array(self.pplotpix)
        x_data = np.arange(0, len(y_data))
        y_data = self.normalize_array(y_data)
        raw_data = self.std_normalize(y_data)

        s = UnivariateSpline(x_data, y_data, s=50)
        yy_data = s(x_data)
        peaks = self.findPeak(yy_data)

        y_data2 = np.array(self.pplotpix2)
        x_data2 = np.arange(0, len(y_data2))
        y_data2 = self.normalize_array(y_data2)
        raw_data2 = self.std_normalize(y_data2)

        s2 = UnivariateSpline(x_data2, y_data2, s=50)
        yy_data2 = s2(x_data2)
        peaks2 = self.findPeak(yy_data2)

        heartrate = 0
        for i in range(0, self.index_per_cycle):
            heartrate += self.rate_per_ten[i]

        heartrate = heartrate / self.index_per_cycle
        print("total_Heart rate=>"+str(heartrate))

        heartrate2 = 0
        for i in range(0, self.index_per_cycle2):
            heartrate2 += self.rate_per_ten2[i]

        heartrate2 = heartrate2 / self.index_per_cycle2
        print("total_Heart rate 2=>"+str(heartrate2))

        if(heartrate < 10):
            heartrate = self.heart_rate_realtime[-1]
        if(heartrate2 < 10):
            heartrate2 = self.heart_rate_realtime2[-1]

        # Get filtered data and find peaks
        atrium_data = self.low_passfilter(yy_data, fps_video)
        ventricle_data = self.low_passfilter(yy_data2, fps_video)
        atrium_data = self.std_normalize(atrium_data)
        ventricle_data = self.std_normalize(ventricle_data)

        inv_atrium_data = -atrium_data
        inv_ventricle_data = -ventricle_data

        interval_1_cof = 0.005
        interval_2_cof = 200
        if real_fps > 35:
            distance_val = int(10*ratio)
            peaks = signal.find_peaks(atrium_data, distance=distance_val) [0]
            peaks2 = signal.find_peaks(ventricle_data, distance=distance_val)[0]
            inv_peaks = signal.find_peaks(inv_atrium_data, distance=distance_val)[0]
            inv_peaks2 = signal.find_peaks(inv_ventricle_data, distance=distance_val)[0]
            interval_1_cof = 0.016
            interval_2_cof = 60
        elif 20 < real_fps and real_fps < 35:
            distance_val = int(80*ratio)
            peaks = signal.find_peaks(atrium_data, distance=distance_val)[0]
            peaks2 = signal.find_peaks(ventricle_data, distance=distance_val)[0]
            inv_peaks = signal.find_peaks(inv_atrium_data, distance=distance_val)[0]
            inv_peaks2 = signal.find_peaks(inv_ventricle_data, distance=distance_val)[0]
            
        # Calculate heartrate and time interval
        _, s_atri_time, s_atri_heart = self.get_heartrate(peaks, real_fps)
        _, s_ventri_time, s_ventri_heart = self.get_heartrate(peaks2, real_fps)

        # Save Raw data

        plt.title("Raw data Waveform", fontsize = 20)
        plt.plot(raw_data, 'bo', label = "Atrium", markersize=3)
        plt.plot(raw_data2, 'ro', label = "Ventricle", markersize=3)
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('raw_data_wave.png')
        plt.cla()
        plt.clf()
        plt.close()
        
        
        # Save splined data
        plt.title("Preprocessed data Waveform", fontsize = 20)
        plt.plot(yy_data, color = "b", label = "Atrium")
        plt.plot(yy_data2, color = "r", label = "Ventricle")
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('peprocessed_data_waveform.png')
        plt.cla()
        plt.clf()
        plt.close()
        

        # Save filtered data
        plt.title("Filtered data Waveform", fontsize = 20)
        plt.plot(atrium_data, color = "b", label = "Atrium")
        plt.plot(peaks, atrium_data[peaks], 'bo', label = "High Peaks of Atrium", alpha = 0.8)
        plt.plot(inv_peaks, -inv_atrium_data[inv_peaks], 'bo', alpha = 0.5, label = "Low Peaks of Atrium")
        plt.plot(ventricle_data, color = "r", label = "Ventricle")
        plt.plot(peaks2, ventricle_data[peaks2], 'ro', label = "High Peaks of Ventricle", alpha = 0.8)
        plt.plot(inv_peaks2, -inv_ventricle_data[inv_peaks2], 'ro', alpha = 0.5, label = "Low Peaks of Ventricle")
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('filtered_data_waveform.png')
        plt.cla()
        plt.clf()
        plt.close()
        

        data_save = {'Heartbeat every 10 seconds': self.heart_rate_realtime, 'Average heart rate(/min)': heartrate}
        df_save = pd.DataFrame(data=data_save)
        df_save.to_csv('result_bps.csv', index=False)

        data_save2 = {'Heartbeat 2 every 10 seconds': self.heart_rate_realtime2, 'Average heart rate 2(/min)': heartrate2}
        df_save2 = pd.DataFrame(data=data_save2)
        df_save2.to_csv('result_bps2.csv', index=False)

        time_list = [len(s_atri_time), len(s_ventri_time), len(s_atri_heart), len(s_ventri_heart), len(peaks), len(peaks2), len(inv_peaks), len(inv_peaks2)]
        time_len = min(time_list) -1 
        s_atri_time = s_atri_time[:time_len]
        s_ventri_time = s_ventri_time[:time_len]
        s_atri_heart = s_atri_heart[:time_len]
        s_ventri_heart = s_ventri_heart[:time_len]
        peaks = peaks[1:time_len+1]
        peaks2 = peaks2[1:time_len+1]
        inv_peaks = inv_peaks[1:time_len+1]
        inv_peaks2 = inv_peaks2[1:time_len+1]

        final = {
            'Time interval (Atrium)':             s_atri_time,
            'Time interval (Ventricle)':          s_ventri_time,
            'Heart rate (Atrium)':                s_atri_heart,
            'Heart rate (Ventricle)':             s_ventri_heart,
            'High peaks (Atrium)':                peaks,
            'High peaks (Ventricle)':             peaks2,
            'Low peaks (Atrium)':                 inv_peaks,
            'Low peaks (Ventricle)':              inv_peaks2
        }

        df_final = pd.DataFrame(data=final)
        df_final.to_csv('result_final.csv', index = False)

        ave_interval = 0
        ave_interval2 = 0
        inv_len = len ( inv_peaks)
        for i in range ( inv_len - 1):
            ave_interval += (inv_peaks[i+1] - inv_peaks[i])*0.005
            ave_interval2 += (inv_peaks2[i+1] - inv_peaks2[i])*0.005

        ave_interval/=(inv_len - 1)
        ave_interval2/=(inv_len-1)
        heartrate=60/ave_interval
        heartrate2=60/ave_interval2
        self.sin_atrium.emit('{:.3f}'.format(heartrate))
        self.sin_ventricle.emit('{:.3f}'.format(heartrate2))

        self.Mbox("Measure result", "Total heartbeat of this animal is " + str(heartrate), 0)
        self.Mbox("Measure result 2", "Total heartbeat of this animal 2 is " + str(heartrate2), 0)

        self.get_interval_1(inv_peaks, inv_peaks2, interval_1_cof)
        self.get_interval_2(inv_peaks, inv_peaks2, interval_2_cof)

        self.Mbox("Success message", "Result saved to 'result_bps.csv' successfully!", 0)
        self.Mbox("Success message", "Result 2 saved to 'result_bps2.csv' successfully!", 0)

        # close all windows
        cv2.destroyAllWindows()

    def result_daphnai(self, fps_video, real_fps, ratio = 1.0):
        y_data = np.array(self.pplotpix)
        x_data = np.arange(0, len(y_data))
        y_data = self.normalize_array(y_data)
        raw_data = self.std_normalize(y_data)

        s = UnivariateSpline(x_data, y_data, s=50)
        yy_data = s(x_data)
        peaks = self.findPeak(yy_data)

        heartrate = 0
        for i in range(0, self.index_per_cycle):
            heartrate += self.rate_per_ten[i]

        heartrate = heartrate / self.index_per_cycle
        print("total_Heart rate=>"+str(heartrate))

        if(heartrate < 10):
            heartrate = self.heart_rate_realtime[-1]

        # Get filtered data and find peaks
        atrium_data = self.low_passfilter(yy_data, fps_video)
        atrium_data = self.std_normalize(atrium_data)

        inv_atrium_data = -atrium_data

        interval_1_cof = 0.005
        if real_fps > 35:
            distance_val = int(10*ratio)
            peaks = signal.find_peaks(atrium_data, distance=distance_val) [0]
            inv_peaks = signal.find_peaks(inv_atrium_data, distance=distance_val)[0]
            interval_1_cof = 0.016
        elif 20 < real_fps and real_fps < 35:
            distance_val = int(80*ratio)
            peaks = signal.find_peaks(atrium_data, distance=distance_val)[0]
            inv_peaks = signal.find_peaks(inv_atrium_data, distance=distance_val)[0]

        # Calculate heartrate and time interval
        _, s_atri_time, s_atri_heart = self.get_heartrate(peaks, real_fps)

        # self.sin_rowData.emit([raw_data, raw_data2])
        # Save Raw data
        plt.clf()
        plt.title("Raw data Waveform", fontsize = 20)
        plt.plot(raw_data, 'bo', label = "Atrium", markersize=3)
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('raw_data_wave.png')
        plt.cla()
        plt.clf()
        plt.close()
        

        # Save splined data
        plt.title("Preprocessed data Waveform", fontsize = 20)
        plt.plot(yy_data, color = "b", label = "Atrium")
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('peprocessed_data_waveform.png')
        plt.cla()
        plt.clf()
        plt.close()
        

        # Save filtered data
        plt.title("Filtered data Waveform", fontsize = 20)
        plt.plot(atrium_data, color = "b", label = "Atrium")
        plt.plot(peaks, atrium_data[peaks], 'bo', label = "High Peaks of Atrium", alpha = 0.8)
        plt.plot(inv_peaks, -inv_atrium_data[inv_peaks], 'bo', alpha = 0.5, label = "Low Peaks of Atrium")
        plt.grid()
        plt.legend(loc = 4)
        plt.savefig('filtered_data_waveform.png')
        plt.cla()
        plt.clf()
        plt.close()
        

        data_save = {'Heartbeat every 10 seconds': self.heart_rate_realtime, 'Average heart rate(/min)': heartrate}
        df_save = pd.DataFrame(data=data_save)
        df_save.to_csv('result_bps.csv', index=False)

        time_list = [len(s_atri_time), len(s_atri_heart), len(peaks), len(inv_peaks)]
        time_len = min(time_list) -1 
        s_atri_time = s_atri_time[:time_len]
        s_atri_heart = s_atri_heart[:time_len]
        peaks = peaks[1:time_len+1]
        inv_peaks = inv_peaks[1:time_len+1]

        final = {
            'Time interval (Atrium)':             s_atri_time,
            'Heart rate (Atrium)':                s_atri_heart,
            'High peaks (Atrium)':                peaks,
            'Low peaks (Atrium)':                 inv_peaks,
        }

        df_final = pd.DataFrame(data=final)

        df_final.to_csv('result_final.csv', index = False)

        self.get_interval_daphnia(inv_peaks, interval_1_cof)

        ave_interval = 0
        inv_len = len ( inv_peaks)
        for i in range ( inv_len - 1):
            ave_interval += (inv_peaks[i+1] - inv_peaks[i])*0.005

        ave_interval/=(inv_len - 1)
        heartrate=60/ave_interval
        self.sin_atrium.emit('{:.3f}'.format(heartrate))

        self.Mbox("Measure result", "Total heartbeat of this animal is " + str(heartrate), 0)

        # close all windows
        cv2.destroyAllWindows()
    
    def main_daphnia(self, frame, fps_video, real_fps, area1, ratio = 1.0):
        
        (x, y, w, h) = area1
        crop_img = frame[y:y + h, x:x + w]
        self.sin_crop_video_1.emit(crop_img)
        
        cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
        self.sin_final_video_frame.emit(frame)

        if(int(self.index) == 0):
            self.first_Image = crop_img

        self.index += 1
        if self.index < fps_video :
            gray_image = cv2.cvtColor(crop_img, cv2.COLOR_BGR2GRAY)
            Binary = cv2.adaptiveThreshold(
                gray_image, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 11, 2)
            cl = np.average(Binary)

            if cl > self.maxcl:
                self.maxcl = cl
                self.first_Image = crop_img
                text = "{}: {}".format(self.index+1, cl)
                cv2.putText(crop_img, text, (10, 20), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (0, 0, 255), 2)
        else:
            cl = self.get_similarity(self.first_Image, crop_img)
            self.pplotpix.append(cl)

            self.sin_result_chart_1.emit(self.myPlot.plot(cl, label='result'))
            
            self.indexFrm += 1
            if self.indexFrm >= fps_video and self.indexFrm % int(fps_video) == 0:
                y_data = np.array(self.pplotpix)
                x_data = np.arange(0, len(y_data))
                y_data = self.normalize_array(y_data)
                s = UnivariateSpline(x_data, y_data, s=50)
                yy_data = s(x_data)
                peaks = self.findPeak(yy_data)

                heatrate = 0
                if(len(peaks) > 1):
                    heatrate = self.get_heartrate(peaks, real_fps)[0]
        
                if(heatrate != 0):
                    self.index_per_cycle += 1
                    self.rate_per_ten.append(heatrate)
                    self.heart_rate_realtime.append(heatrate)
                    self.sin_atrium.emit('{:.3f}'.format(heatrate))
                    self.sin_ventricle.emit('0.00')

    def run(self):

        while True:
            # print(self.running_flag)
            if self.running_flag == 1:
                cap = cv2.VideoCapture(self.video_file_name)
                fps_video = 6
                fcount_video = cap.get(cv2.CAP_PROP_FRAME_COUNT)
                real_fps = cap.get(cv2.CAP_PROP_FPS)
                print("Video Frame Count : ", fcount_video)
                print("video fps :", cap.get(cv2.CAP_PROP_FPS))
                
                ratio = 1.0
                if self.fish_type == "fish":
                    ratio = 1.0
                elif self.fish_type == 'daphnia':
                    ratio = 0.3
                elif self.fish_type == 'daphnia_35':
                    ratio = 0.33
                elif self.fish_type == 'daphnia_IMI':
                    ratio =1.5
                while cap.isOpened() and self.running_flag:
                    ret, frame = cap.read()
                    self.sin_frame_count.emit([self.index, fcount_video])
                    if not ret:
                        self.running_flag = 0

                        print ('end of the video file...')
                        break
                    if self.fish_type == 'fish':
                        
                        self.main_fish(frame, fps_video, real_fps, self.roi_rectAngle_1, self.roi_rectAngle_2, ratio)
                    else:
                        self.main_daphnia(frame, fps_video, real_fps, self.roi_rectAngle_1, ratio)
                
                    time.sleep(0.01)
                try:
                    if self.fish_type == 'fish':
                        self.result_fish(fps_video, real_fps, ratio)
                        self.sin_end_frame.emit(1)
                    else:
                        self.result_daphnai(fps_video, real_fps, ratio)
                        self.sin_end_frame.emit(1)
                except:
                    self.sin_end_frame.emit(1)
                    self.running_flag = 0
                    print('result data save error')
                    pass
                cap.release()
                    




